@@ -96,10 +96,17 @@ pub(crate) fn get_global_keymaps() -> Vec<Keymap> {
             Input::new(KeyCode::Char('q'), KeyModifiers::NONE),
             UICommand::Quit,
         ),
+        // Char '?' isn't recognized on windows
+        #[cfg(not(target_os = "windows"))]
         Keymap::new(
             Input::new(KeyCode::Char('?'), KeyModifiers::NONE),
             UICommand::ShowHelp,
         ),
+        #[cfg(target_os = "windows")]
+        Keymap::new(
+            Input::new(KeyCode::Char('h'), KeyModifiers::NONE),
+            UICommand::ShowHelp,
+        ),
         Keymap::new(
             Input::new(KeyCode::Char('l'), KeyModifiers::CONTROL),
             UICommand::CycleFocusedControlForward,
@@ -339,9 +346,16 @@ pub fn get_multi_select_keymaps() -> Vec<Keymap> {
             Input::new(KeyCode::Char('>'), KeyModifiers::NONE),
             UICommand::MulSelExportEntries,
         ),
+        // Char '?' isn't recognized on windows
+        #[cfg(not(target_os = "windows"))]
         Keymap::new(
             Input::new(KeyCode::Char('?'), KeyModifiers::NONE),
             UICommand::ShowHelp,
         ),
+        #[cfg(target_os = "windows")]
+        Keymap::new(
+            Input::new(KeyCode::Char('h'), KeyModifiers::NONE),
+            UICommand::ShowHelp,
+        ),
     ]
 }
