@@ -18,19 +18,22 @@ pub fn exec_select_prev_entry<D: DataProvider>(
     if ui_components.has_unsaved() {
         ui_components.show_unsaved_msg_box(Some(UICommand::SelectedPrevEntry));
     } else {
-        select_prev_entry(ui_components, app);
+        select_prev_entry(1, ui_components, app);
     }
 
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
-fn select_prev_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+fn select_prev_entry<D: DataProvider>(
+    step: usize,
+    ui_components: &mut UIComponents,
+    app: &mut App<D>,
+) {
     let prev_id = ui_components
         .entries_list
         .state
         .selected()
-        .and_then(|index| index.checked_sub(1))
+        .map(|index| index.saturating_sub(step))
         .and_then(|prev_index| {
             app.get_active_entries()
                 .nth(prev_index)
@@ -51,9 +54,9 @@ pub async fn continue_select_prev_entry<'a, D: DataProvider>(
         MsgBoxResult::Ok | MsgBoxResult::Cancel => {}
         MsgBoxResult::Yes => {
             exec_save_entry_content(ui_components, app).await?;
-            select_prev_entry(ui_components, app);
+            select_prev_entry(1, ui_components, app);
         }
-        MsgBoxResult::No => select_prev_entry(ui_components, app),
+        MsgBoxResult::No => select_prev_entry(1, ui_components, app),
     }
 
     Ok(HandleInputReturnType::Handled)
@@ -66,22 +69,26 @@ pub fn exec_select_next_entry<D: DataProvider>(
     if ui_components.has_unsaved() {
         ui_components.show_unsaved_msg_box(Some(UICommand::SelectedNextEntry));
     } else {
-        select_next_entry(ui_components, app);
+        select_next_entry(1, ui_components, app);
     }
 
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
-fn select_next_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+fn select_next_entry<D: DataProvider>(
+    step: usize,
+    ui_components: &mut UIComponents,
+    app: &mut App<D>,
+) {
     let next_id = ui_components
         .entries_list
         .state
         .selected()
-        .and_then(|index| index.checked_add(1))
+        .and_then(|index| index.checked_add(step))
         .and_then(|next_index| {
             app.get_active_entries()
                 .nth(next_index)
+                .or_else(|| app.get_active_entries().next_back())
                 .map(|entry| entry.id)
         });
 
@@ -99,29 +106,31 @@ pub async fn continue_select_next_entry<'a, D: DataProvider>(
         MsgBoxResult::Ok | MsgBoxResult::Cancel => {}
         MsgBoxResult::Yes => {
             exec_save_entry_content(ui_components, app).await?;
-            select_next_entry(ui_components, app);
+            select_next_entry(1, ui_components, app);
         }
-        MsgBoxResult::No => select_next_entry(ui_components, app),
+        MsgBoxResult::No => select_next_entry(1, ui_components, app),
     }
 
     Ok(HandleInputReturnType::Handled)
 }
 
-pub fn exec_create_entry(ui_components: &mut UIComponents) -> CmdResult {
+pub fn exec_create_entry<D: DataProvider>(
+    ui_components: &mut UIComponents,
+    app: &App<D>,
+) -> CmdResult {
     if ui_components.has_unsaved() {
         ui_components.show_unsaved_msg_box(Some(UICommand::CreateEntry));
     } else {
-        create_entry(ui_components);
+        create_entry(ui_components, app);
     }
 
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
-pub fn create_entry(ui_components: &mut UIComponents) {
+pub fn create_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &App<D>) {
     ui_components
         .popup_stack
-        .push(Popup::Entry(Box::new(EntryPopup::new_entry())));
+        .push(Popup::Entry(Box::new(EntryPopup::new_entry(&app.settings))));
 }
 
 pub async fn continue_create_entry<'a, D: DataProvider>(
@@ -133,9 +142,9 @@ pub async fn continue_create_entry<'a, D: DataProvider>(
         MsgBoxResult::Ok | MsgBoxResult::Cancel => {}
         MsgBoxResult::Yes => {
             exec_save_entry_content(ui_components, app).await?;
-            create_entry(ui_components);
+            create_entry(ui_components, app);
         }
-        MsgBoxResult::No => create_entry(ui_components),
+        MsgBoxResult::No => create_entry(ui_components, app),
     }
 
     Ok(HandleInputReturnType::Handled)
@@ -154,7 +163,6 @@ pub fn exec_edit_current_entry<D: DataProvider>(
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
 fn edit_current_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
     if let Some(entry) = app.get_current_entry() {
         ui_components
@@ -231,7 +239,6 @@ pub fn exec_export_entry_content<D: DataProvider>(
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
 pub fn export_entry_content<D: DataProvider>(ui_components: &mut UIComponents, app: &App<D>) {
     if let Some(entry) = app.get_current_entry() {
         match ExportPopup::create_entry_content(entry, app) {
@@ -348,7 +355,6 @@ pub fn exec_show_filter<D: DataProvider>(
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
 fn show_filter<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
     let tags = app.get_all_tags();
     ui_components
@@ -398,7 +404,6 @@ pub fn exec_show_fuzzy_find<D: DataProvider>(
     Ok(HandleInputReturnType::Handled)
 }
 
-#[inline]
 fn show_fuzzy_find<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
     let entries: HashMap<u32, String> = app
         .get_active_entries()
@@ -428,3 +433,78 @@ pub async fn continue_fuzzy_find<'a, D: DataProvider>(
 
     Ok(HandleInputReturnType::Handled)
 }
+
+pub fn exec_toggle_full_screen_mode<D: DataProvider>(app: &mut App<D>) -> CmdResult {
+    app.state.full_screen = !app.state.full_screen;
+    Ok(HandleInputReturnType::Handled)
+}
+
+pub fn exec_show_sort_options<D: DataProvider>(
+    ui_components: &mut UIComponents,
+    app: &mut App<D>,
+) -> CmdResult {
+    if ui_components.has_unsaved() {
+        ui_components.show_unsaved_msg_box(Some(UICommand::ShowSortOptions));
+    } else {
+        show_sort_options(ui_components, app);
+    }
+
+    Ok(HandleInputReturnType::Handled)
+}
+
+fn show_sort_options<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+    ui_components
+        .popup_stack
+        .push(Popup::Sort(Box::new(SortPopup::new(&app.state.sorter))));
+}
+
+pub async fn continue_show_sort_options<'a, D: DataProvider>(
+    ui_components: &mut UIComponents<'a>,
+    app: &mut App<D>,
+    msg_box_result: MsgBoxResult,
+) -> CmdResult {
+    match msg_box_result {
+        MsgBoxResult::Ok | MsgBoxResult::Cancel => {}
+        MsgBoxResult::Yes => {
+            exec_save_entry_content(ui_components, app).await?;
+            show_sort_options(ui_components, app);
+        }
+        MsgBoxResult::No => {
+            // Discard the current content explicitly because it doesn't get discarded if the sort
+            // was cancelled which could confuse the users
+            discard_current_content(ui_components, app);
+
+            show_sort_options(ui_components, app);
+        }
+    }
+
+    Ok(HandleInputReturnType::Handled)
+}
+
+pub fn go_to_top_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+    let top_id = app.get_active_entries().next().map(|entry| entry.id);
+
+    if top_id.is_some() {
+        ui_components.set_current_entry(top_id, app);
+    }
+}
+
+pub fn go_to_bottom_entry<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+    let top_id = app.get_active_entries().next_back().map(|entry| entry.id);
+
+    if top_id.is_some() {
+        ui_components.set_current_entry(top_id, app);
+    }
+}
+
+pub fn page_up_entries<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+    let step = app.settings.get_scroll_per_page();
+
+    select_prev_entry(step, ui_components, app);
+}
+
+pub fn page_down_entries<D: DataProvider>(ui_components: &mut UIComponents, app: &mut App<D>) {
+    let step = app.settings.get_scroll_per_page();
+
+    select_next_entry(step, ui_components, app);
+}
